import time
import unittest

from threading import Thread
from rpyc.utils.registry import TCPRegistryServer, TCPRegistryClient
from rpyc.utils.registry import UDPRegistryServer, UDPRegistryClient


PRUNING_TIMEOUT = 5


class BaseRegistryTest(object):
    def _get_server(self):
        raise NotImplementedError

    def _get_client(self):
        raise NotImplementedError

    def setUp(self):
        self.server = self._get_server()
        self.server.logger.quiet = True
        self.server_thread = Thread(target=self.server.start)
        self.server_thread.setDaemon(True)
        self.server_thread.start()
        time.sleep(0.1)

    def tearDown(self):
        self.server.close()
        self.server_thread.join()

    def test_api(self):
        c = self._get_client()
        c.logger.quiet = True
        c.register(("FOO",), 12345)
        c.register(("FOO",), 45678)
        res = c.discover("FOO")
        expected = (12345, 45678)
        self.assertEqual(set(p for _, p in res), set(expected))
        c.unregister(12345)
        res = c.discover("FOO")
        expected = (45678,)
        self.assertEqual(set(p for _, p in res), set(expected))

    def test_pruning(self):
        c = self._get_client()
        c.logger.quiet = True
        c.register(("BAR",), 17171)

        time.sleep(1)
        res = c.discover("BAR")
        self.assertEqual(set(p for _, p in res), set((17171,)))

        time.sleep(PRUNING_TIMEOUT)
        res = c.discover("BAR")
        self.assertEqual(res, ())


class TestTcpRegistry(BaseRegistryTest, unittest.TestCase):
    def _get_server(self):
        return TCPRegistryServer(pruning_timeout=PRUNING_TIMEOUT)

    def _get_client(self):
        return TCPRegistryClient("localhost")

class TestUdpRegistry(BaseRegistryTest, unittest.TestCase):
    def _get_server(self):
        return UDPRegistryServer(pruning_timeout=PRUNING_TIMEOUT)

    def _get_client(self):
        return UDPRegistryClient()


if __name__ == "__main__":
    unittest.main()





import rpyc
import os
import time
from threading import Thread

class FileMonitorService(rpyc.Service):
    class exposed_FileMonitor(object):
        def __init__(self, filename, callback, interval = 1):
            self.filename = filename
            self.interval = interval
            self.last_stat = None
            self.callback = rpyc.async(callback)   # make the callback async
            self.active = True
            self.thread = Thread(target = self.work)
            self.thread.start()
        def exposed_stop(self):
            self.active = False
            self.thread.join()
        def work(self):
            while self.active:
                stat = os.stat(self.filename)
                if self.last_stat is not None and self.last_stat != stat:
                    self.callback(self.last_stat, stat)
                self.last_stat = stat
                time.sleep(self.interval)


if __name__ == "__main__":
    from rpyc.utils.server import ThreadedServer
    ThreadedServer(FileMonitorService, port = 18871).start()




#!/usr/bin/env python
from __future__ import with_statement
import daemon
import lockfile
import sys
import signal
from rpyc.utils.server import ThreadedServer, ForkingServer
from rpyc.core.service import SlaveService
from rpyc.lib import setup_logger
try:
    from configparser import ConfigParser
except ImportError:
    from ConfigParser import ConfigParser

server = None

def start():
    global server

    conf = ConfigParser()
    conf.read('rpycd.conf')

    mode = conf.get("rpycd", "mode").lower()
    if mode == "threaded":
        factory = ThreadedServer
    elif mode == "forking":
        factory = ForkingServer
    else:
        raise ValueError("Invalid mode %r" % (mode,))

    setup_logger(conf.getboolean("rpycd", "quiet"), conf.get("rpycd", "logfile"))

    server = factory(SlaveService, hostname = conf.get("rpycd", "host"),
        port = conf.getint("rpycd", "port"), reuse_addr = True)
    server.start()

def reload(*args):
    server.close()
    start()

def stop(*args):
    server.close()
    sys.exit()


if __name__ == "__main__":
    with daemon.DaemonContext(
            pidfile = lockfile.FileLock('/var/run/rpydc.pid'),
            signal_map = {signal.SIGTERM: stop, signal.SIGHUP: reload}):
        start()




import sys
import rpyc
from plumbum import SshMachine
from rpyc.utils.zerodeploy import DeployedServer
from rpyc.utils.splitbrain import splitbrain

mach = SshMachine("192.168.1.117")

print sys.platform

with DeployedServer(mach) as dep:
    conn = dep.classic_connect()
    print conn.modules.sys.platform

    try:
        import posix
    except ImportError as ex:
        print ex

    with splitbrain(conn):
        import posix
        print posix.stat("/boot")

    print posix






import rpyc 

conn = rpyc.connect ("localhost", 18861) 

store = conn.root.store 
Order = conn.root.Order 
estrella_levante = conn.root.estrella_levante #beer supplier 
client = conn.root.client 
Product = conn.root.Product 

def show_reserved_products (product):  #callback 
    print 'You have reserved %s' %  product 

#Product.reserveds is the py-notify signal that is thrown when you 
# output a product from thee store 
conn.root.connect (Product.reserveds, show_reserved_products) 

p = store.output ('quinto', estrella_levante) #store.output gives me a Product object
print "@@", type(p), p
order = Order (client) 
order.add_product (p)      #  <---------- this is the interesting moment 
                           #  we pass as argument a Product object 

conn.close ()




import socket
import select
import ssl
import threading
import time

files = [open("/tmp/rpyc-test-%d" % (i,), "w") for i in range(1000)]
sockets = [socket.socket() for i in range(100)]

listener = socket.socket()
assert listener.fileno() > 1024

listener.bind(("localhost", 13388))
listener.listen(10)

def handle_sock(s):
    s2 = ssl.wrap_socket(s, server_side = True, keyfile = 'cert.key', certfile = 'cert.crt', 
        cert_reqs = ssl.CERT_NONE, ca_certs = None, ssl_version = ssl.PROTOCOL_TLSv1)
    select.select([s2], [], [], 1)
    for i in range(3):
        data = s2.recv(1000)
        s2.send(data)
    time.sleep(1)
    #s2.close()

while True:
    s, _ = listener.accept()
    assert s.fileno() > 1024
    t = threading.Thread(target = handle_sock, args = (s,))
    t.start()






from __future__ import with_statement
import rpyc
import unittest

from contextlib import contextmanager

on_context_enter = False
on_context_exit = False

class MyService(rpyc.Service):
    @contextmanager
    def exposed_context(self, y):
        global on_context_enter, on_context_exit
        on_context_enter = True
        try:
            yield 17 + y
        finally:
            on_context_exit = True


class TestContextManagers(unittest.TestCase):
    def setUp(self):
        self.conn = rpyc.connect_thread(remote_service=MyService)

    def tearDown(self):
        self.conn.close()
    
    def test_context(self):
        with self.conn.root.context(3) as x:
            print( "entering test" )
            self.assertTrue(on_context_enter)
            self.assertFalse(on_context_exit)
            print( "got past context enter" )
            self.assertEqual(x, 20)
            print( "got past x=20" )
        self.assertTrue(on_context_exit)
        print( "got past on_context_exit" )


if __name__ == "__main__":
    unittest.main()




from __future__ import with_statement
import subprocess
import sys
import os
import rpyc
import unittest
from rpyc.utils.teleportation import export_function, import_function
from rpyc.utils.classic import teleport_function


def b(st):
    if sys.version_info[0] >= 3:
        return bytes(st, "latin-1")
    else:
        return st

def f(a):
    def g(b):
        return a + int(b)
    return g

def h(a):
    import os
    return a * os.getpid()


class TeleportationTest(unittest.TestCase):
    def setUp(self):
        server_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "bin", "rpyc_classic.py")
        self.proc = subprocess.Popen([sys.executable, server_file, "--mode=oneshot", "--host=localhost", "-p0"],
            stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        line = self.proc.stdout.readline().strip()
        if not line:
            print (self.proc.stderr.read())
            self.fail("server failed to start")
        self.assertEqual(line, b("rpyc-oneshot"), "server failed to start")
        host, port = self.proc.stdout.readline().strip().split(b("\t"))
        self.conn = rpyc.classic.connect(host, int(port))

    def tearDown(self):
        self.conn.close()

    def test(self):
        exp = export_function(f)
        f2 = import_function(exp)
        self.assertEqual(f(6)(7), f2(6)(7))

        # HACK: needed so the other side could import us (for globals)
        mod = self.conn.modules.types.ModuleType(__name__)
        self.conn.modules.sys.modules[__name__] = mod
        mod.__builtins__ = self.conn.builtins

        h2 = teleport_function(self.conn, h)
        self.assertNotEqual(h(7), h2(7))



if __name__ == "__main__":
    unittest.main()






import rpyc
import time

count = 0

def callbackFunc(x):
    global count
    count += 1
    print x, time.time()

if __name__ == "__main__":
    conn = rpyc.connect("localhost", 12000)
    #rpyc.BgServingThread.SERVE_INTERVAL = 0.01
    rpyc.BgServingThread.SLEEP_INTERVAL = 0.0001
    bgsrv = rpyc.BgServingThread(conn)

    test = conn.root.RemoteCallbackTest(callbackFunc)
    print test
    test.start()
    print "doing other things while the callback is being called"
    while count < 100:
        time.sleep(0.1)
    print "done"





import threading
import rpyc


def run_something(callback):
    for i in range(100):
        callback(i)

class MyService(rpyc.Service):
    def on_connect(self):
        print "hi", self._conn._config["endpoints"][1]
    def on_disconnect(self):
        print "bye", self._conn._config["endpoints"][1]
    
    class exposed_RemoteCallbackTest(object):
        def __init__(self, callback):
            self.callback = callback
        def start(self):
            thd = threading.Thread(target = run_something, args = (self.callback,))
            thd.start()


if __name__ == "__main__":
    from rpyc.utils.server import ThreadedServer
    myServerObj = ThreadedServer(MyService, port=12000, protocol_config={"allow_public_attrs":True})
    myServerObj.start()




import rpyc


class RemoteFSService(rpyc.Service):
    def open(self, filename, mode = "r"):
        pass
    def listdir(self, path):
        pass
    def mkdir(self, path):
        pass
    def stat(self, path):
        pass
    def rmdir(self, path):
        pass




from rpyc.core.stream import SocketStream, TunneledSocketStream, PipeStream
from rpyc.core.channel import Channel
from rpyc.core.protocol import Connection
from rpyc.core.netref import BaseNetref
from rpyc.core.async import AsyncResult, AsyncResultTimeout
from rpyc.core.service import Service, VoidService, SlaveService
from rpyc.core.vinegar import GenericException



import rpyc
import time

c = rpyc.classic.connect("localhost")
t = rpyc.BgServingThread(c)

start = time.time()
for i in range(100):
    c.execute("newObj = %d" % (i))
stop = time.time()
print "added %d simple objects one by one, %f seconds" % (100, stop - start)

t.stop()




import math
import time

import rpyc
import unittest

on_connect_called = False
on_disconnect_called = False

class MyMeta(type):
    def spam(self):
        return self.__name__ * 5

class MyClass(object):
    __metaclass__ = MyMeta

if not isinstance(MyMeta, MyMeta):
    # python 3 compatibility
    MyClass = MyMeta(MyClass.__name__, MyClass.__bases__, dict(MyClass.__dict__))

class MyService(rpyc.Service):
    def on_connect(self):
        global on_connect_called
        on_connect_called = True

    def on_disconnect(self):
        global on_disconnect_called
        on_disconnect_called = True

    def exposed_distance(self, p1, p2):
        x1, y1 = p1
        x2, y2 = p2
        return math.sqrt((x2-x1)**2 + (y2-y1)**2)

    def exposed_getlist(self):
        return [1, 2, 3]

    def foobar(self):
        assert False

    def exposed_getmeta(self):
        return MyClass()


class TestCustomService(unittest.TestCase):
    config = {}

    def setUp(self):
        global on_connect_called
        self.conn = rpyc.connect_thread(remote_service=MyService)
        self.conn.root # this will block until the service is initialized,
        # so we can be sure on_connect_called is True by that time
        self.assertTrue(on_connect_called)
        on_connect_called = False

    def tearDown(self):
        global on_disconnect_called
        self.conn.close()
        time.sleep(0.5) # this will wait a little, making sure
        # on_disconnect_called is already True
        self.assertTrue(on_disconnect_called)
        on_disconnect_called = False

    def test_aliases(self):
        print( "service name: %s" % (self.conn.root.get_service_name(),) )

    def test_distance(self):
        assert self.conn.root.distance((2,7), (5,11)) == 5

    def test_attributes(self):
        self.conn.root.distance
        self.conn.root.exposed_distance
        self.conn.root.getlist
        self.conn.root.exposed_getlist
        # this is not an exposed attribute:
        self.assertRaises(AttributeError, lambda: self.conn.root.foobar()) 

    def test_safeattrs(self):
        x = self.conn.root.getlist()
        #self.require(x == [1, 2, 3]) -- can't compare remote objects, sorry
        #self.require(x * 2 == [1, 2, 3, 1, 2, 3])
        self.assertEqual([y*2 for y in x], [2, 4, 6])

    def test_metaclasses(self):
        x = self.conn.root.getmeta()
        print( x )


if __name__ == "__main__":
    unittest.main()





import rpyc

class AService(rpyc.Service):
    class exposed_A(object):
        @classmethod
        def exposed_foo(cls, a, b):
            return 17 * a + b

if __name__ == "__main__":
    with rpyc.connect_thread(remote_service = AService) as conn:
        print( conn.root.A.foo(1, 2))




import rpyc
import time
import threading
import sys
import os
import unittest
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService
from plumbum import SshMachine


class Test_Ssh(unittest.TestCase):
    def setUp(self):
        if sys.platform == "win32":
            self.server = None
            os.environ["HOME"] = os.path.expanduser("~")
            self.remote_machine = SshMachine("localhost")
        else:
            # assume "ssh localhost" is configured to run without asking for password 
            self.server = ThreadedServer(SlaveService, hostname = "localhost", 
                ipv6 = False, port = 18888, auto_register=False)
            t = threading.Thread(target=self.server.start)
            t.setDaemon(True)
            t.start()
            time.sleep(0.5)
            self.remote_machine = SshMachine("localhost")

    def tearDown(self):
        if self.server:
            self.server.close()

    def test_simple(self):
        conn = rpyc.classic.ssh_connect(self.remote_machine, 18888)
        print( "server's pid =", conn.modules.os.getpid())
        conn.modules.sys.stdout.write("hello over ssh\n")
        conn.modules.sys.stdout.flush()

    def test_connect(self):
        conn2 = rpyc.ssh_connect(self.remote_machine, 18888, service=SlaveService)
        conn2.modules.sys.stdout.write("hello through rpyc.ssh_connect()\n")
        conn2.modules.sys.stdout.flush()

if __name__ == "__main__":
    unittest.main()





import rpyc
import unittest
import time
from rpyc.utils.server import ThreadedServer
from threading import Thread


class MyClass(object):
    def foo(self):
        return "foo"
    def bar(self):
        return "bar"
    def spam(self):
        return "spam"

class YourClass(object):
    def lala(self):
        return MyClass()
    def baba(self):
        return "baba"
    def gaga(self):
        return "gaga"

try:
    long
except NameError:
    long = int
    unicode = str

try:
    bytes
except NameError:
    bytes = str

class Protector(object):
    def __init__(self, safetypes = (int, list, bool, tuple, str, float, long, unicode, bytes)):
        self._safetypes = set(safetypes)
        self._typereg = {}
    def register(self, typ, attrs):
        self._typereg[typ] = frozenset(attrs)
    def wrap(self, obj):
        class Restrictor(object):
            def __call__(_, *args, **kwargs):
                return self.wrap(obj(*args, **kwargs))
            def _rpyc_getattr(_, name):
                if type(obj) not in self._safetypes:
                    attrs = self._typereg.get(type(obj), ())
                    if name not in attrs:
                        raise AttributeError(name)
                obj2 = getattr(obj, name)
                return self.wrap(obj2)
            __getattr__ = _rpyc_getattr
        return Restrictor()

class MyService(rpyc.Service):
    def exposed_get_one(self):
        return rpyc.restricted(MyClass(), ["foo", "bar"])
    
    def exposed_get_two(self):
        protector = Protector()
        protector.register(MyClass, ["foo", "spam"])
        protector.register(YourClass, ["lala", "baba"])
        return protector.wrap(YourClass())

class TestRestricted(unittest.TestCase):
    def setUp(self):
        self.server = ThreadedServer(MyService, port = 0)
        self.thd = Thread(target = self.server.start)
        self.thd.start()
        time.sleep(1)
        self.conn = rpyc.connect("localhost", self.server.port)

    def tearDown(self):
        self.conn.close()
        self.server.close()
        self.thd.join()

    def test_restricted(self):
        obj = self.conn.root.get_one()
        self.assertEqual(obj.foo(), "foo")
        self.assertEqual(obj.bar(), "bar")
        self.assertRaises(AttributeError, lambda: obj.spam)

#    def test_type_protector(self):
#        obj = self.conn.root.get_two()
#        assert obj.baba() == "baba"
#        try:
#            obj.gaga()
#        except AttributeError:
#            pass
#        else:
#            assert False, "expected an attribute error!"
#        obj2 = obj.lala()
#        assert obj2.foo() == "foo"
#        assert obj2.spam() == "spam"
#        try:
#            obj.bar()
#        except AttributeError:
#            pass
#        else:
#            assert False, "expected an attribute error!"
#        


if __name__ == "__main__":
    unittest.main()







import rpyc


class MyService(rpyc.Service):
    def exposed_foo(self):
        return 18



if __name__ == "__main__":
    from rpyc.utils.server import ThreadedServer
    from rpyc.utils.authenticators import SSLAuthenticator
    server = ThreadedServer(MyService, port = 13388, 
        authenticator = SSLAuthenticator("cert.key", "cert.crt"),
    )
    server.start()



import rpyc

#
# with explicit closing
#
for i in range(5000):
    #if i % 100 == 0:
    #    print i
    c = rpyc.ssl_connect("localhost", 13388, keyfile = "cert.key", certfile = "cert.crt")
    print i, c.fileno()
    #c = rpyc.connect("localhost", 13388)
    assert c.root.foo() == 18
    c.close()

print
print "finished (1/2)"

#
# without explicit closing
#
for i in range(5000):
    if i % 100 == 0:
        print i
    c = rpyc.ssl_connect("localhost", 13388, keyfile = "cert.key", certfile = "cert.crt")
    #c = rpyc.connect("localhost", 13388)
    assert c.root.foo() == 18
    #c.close()

print
print "finished (2/2)"




import rpyc
import gc

c = rpyc.classic.connect("localhost")
c.execute("""class Foo(object):
    def __init__(self, name):
        self.name = name
    def __del__(self):
        print "%s deleted" % (self.name,)
""")

f1 = c.namespace["Foo"]("f1")
f2 = c.namespace["Foo"]("f2")
f3 = c.namespace["Foo"]("f3")

del f1
del f2
gc.collect()

raw_input()
c.close()




import rpyc
import os
from contextlib import contextmanager

class Number(object):
    def __init__(self,number):
        self.number=number

def f(number):
    print( number.number)


@contextmanager
def ASYNC(func):
    wrapper = rpyc.async(func)
    yield wrapper

if __name__ == "__main__":
    conn = rpyc.classic.connect("localhost")
    conn.modules.sys.path.append(os.path.dirname(__file__))

    mod = conn.modules["issue14"]
    n = Number(999)
    #f = rpyc.async(mod.f)(n)
    #print( f )
    #print( f.value )

    f2 = rpyc.async(mod.f)
    res = f2(n)
    print res.value
    
    with ASYNC(mod.f) as f2:
        res = f2(n)
        print res.value





import socket
import ssl

for i in range(5000):
    if i % 100 == 0:
        print i
    sock = socket.socket()
    sock.connect(("localhost", 13388))
    sock2 = ssl.wrap_socket(sock, server_side = False, keyfile = 'cert.key', 
        certfile = 'cert.crt', cert_reqs = ssl.CERT_NONE, ca_certs = None, 
        ssl_version = ssl.PROTOCOL_TLSv1)
    
    for text in ["hello world", "foobar", "spam and eggs"]:
        sock2.send(text)
        data = sock2.recv(1000)
        assert data == text
    #sock.close()




import rpyc
import os
import threading
import unittest
import time
from rpyc.utils.authenticators import SSLAuthenticator
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService
from nose import SkipTest

try:
    import ssl #@UnusedImport
except ImportError:
    raise SkipTest("requires ssl")


class Test_SSL(unittest.TestCase):
    '''
    created key like that
    http://www.akadia.com/services/ssh_test_certificate.html

    openssl req -newkey rsa:1024 -nodes -keyout mycert.pem -out mycert.pem
    '''

    def setUp(self):
        self.key = os.path.join( os.path.dirname(__file__) , "server.key")
        self.cert =  os.path.join( os.path.dirname(__file__) , "server.crt")
        print( self.cert, self.key )

        authenticator = SSLAuthenticator(self.key, self.cert)
        self.server = ThreadedServer(SlaveService, port = 18812,
            auto_register=False, authenticator = authenticator)
        self.server.logger.quiet = False
        t = threading.Thread(target=self.server.start)
        t.start()
        time.sleep(1)

    def tearDown(self):
        self.server.close()

    def test_ssl_conenction(self):
        c = rpyc.classic.ssl_connect("localhost", port = 18812,
            keyfile=self.key, certfile=self.cert)
        print( repr(c) )
        print( c.modules.sys )
        print( c.modules["xml.dom.minidom"].parseString("<a/>") )
        c.execute("x = 5")
        self.assertEqual(c.namespace["x"], 5)
        self.assertEqual(c.eval("1+x"), 6)
        c.close()

if __name__ == "__main__":
    unittest.main()




import rpyc
from rpyc.utils.splitbrain import splitbrain, localbrain
import traceback
import sys


c = rpyc.classic.connect("localhost")
import os
pid1 = os.getpid()

with open("tmp.txt", "w") as f:
    f.write("foobar")

with splitbrain(c):
    pid2 = os.getpid()
    assert pid1 != pid2
    import email
    print (email)
    import os as os2
    pid3 = os2.getpid()
    assert pid2 == pid3
    
    assert not os.path.exists("tmp.txt")
    
    with localbrain():
        with open("tmp.txt", "r") as f:
            assert f.read() == "foobar"
        pid4 = os.getpid()
        assert pid4 == pid1
    
    try:
        open("tmp.txt", "r")
    except IOError as ex:
        #print(type(ex), repr(ex))
        with localbrain():
            x = ("".join(traceback.format_exception(*sys.exc_info())))
            print(len(x))
    else:
        assert False, "expected an IOError"

pid5 = os.getpid()
assert pid5 == pid1

print ("done")



#!/usr/bin/env python
import sys
import os

try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup

if sys.version_info < (2, 5):
    sys.exit("requires python 2.5 and up")

here = os.path.dirname(__file__)
exec(open(os.path.join(here, 'rpyc', 'version.py')).read())

setup(name = "rpyc",
    version = version_string, #@UndefinedVariable
    description = "Remote Python Call (RPyC), a transparent and symmetric RPC library",
    author = "Tomer Filiba",
    author_email = "tomerfiliba@gmail.com",
    maintainer_email = "rpyc@googlegroups.com",
    license = "MIT",
    url = "http://rpyc.readthedocs.org",
    packages = [
        'rpyc',
        'rpyc.core',
        'rpyc.lib',
        'rpyc.experimental',
        'rpyc.utils',
    ],
    scripts = [
        os.path.join(here, "bin", "rpyc_classic.py"),
        os.path.join(here, "bin", "rpyc_registry.py"),
    ],
    tests_require = ['nose'],
    test_suite = 'nose.collector',
    install_requires = ["plumbum"],
#    entry_points = dict(
#        console_scripts = [
#            "rpyc_classic = rpyc.scripts.rpyc_classic:main",
#            "rpyc_registry = rpyc.scretips.rpyc_registry:main",
#        ],
#    ),
    platforms = ["POSIX", "Windows"],
    use_2to3 = False,
    zip_safe = False,
    long_description = open(os.path.join(here, "README.rst"), "r").read(),
    classifiers = [
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python :: 2.6",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.3",
        "Programming Language :: Python :: 3.4",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Topic :: Internet",
        "Topic :: Software Development :: Libraries :: Python Modules",
        "Topic :: Software Development :: Object Brokering",
        "Topic :: Software Development :: Testing",
        "Topic :: System :: Clustering",
        "Topic :: System :: Distributed Computing",
        "Topic :: System :: Monitoring",
        "Topic :: System :: Networking",
        "Topic :: System :: Systems Administration",
    ],
)




import rpyc
import threading
import socket
import unittest
import time
from rpyc.utils.server import ThreadedServer
from rpyc import SlaveService
from nose import SkipTest

#if not getattr(socket, "has_ipv6", False):
# travis: "Network is unreachable", https://travis-ci.org/tomerfiliba/rpyc/jobs/108231239#L450
raise SkipTest("requires IPv6")


class Test_IPv6(unittest.TestCase):
    def setUp(self):
        self.server = ThreadedServer(SlaveService, port = 0, ipv6 = True)
        self.server.logger.quiet = True
        self.thd = threading.Thread(target = self.server.start)
        self.thd.start()
        time.sleep(1)

    def tearDown(self):
        self.server.close()
        self.thd.join()

    def test_ipv6_conenction(self):
        c = rpyc.classic.connect("::1", port = self.server.port, ipv6 = True)
        print( repr(c) )
        print( c.modules.sys )
        print( c.modules["xml.dom.minidom"].parseString("<a/>") )
        c.execute("x = 5")
        self.assertEqual(c.namespace["x"], 5)
        self.assertEqual(c.eval("1+x"), 6)
        c.close()


if __name__ == "__main__":
    unittest.main()






from __future__ import with_statement 
import unittest
import sys
from plumbum import SshMachine
from rpyc.utils.zerodeploy import DeployedServer


class TestDeploy(unittest.TestCase):
    def test_deploy(self):
        rem = SshMachine("localhost")
        SshMachine.python = rem[sys.executable]
        with DeployedServer(rem) as dep:
            conn = dep.classic_connect()
            print (conn.modules.sys)
            func = conn.modules.os.getcwd
            print (func())
        
        try:
            func()
        except EOFError:
            pass
        else:
            self.fail("expected an EOFError")
    
    def test_deploy_paramiko(self):
        try:
            import paramiko     # @UnusedImport
        except Exception:
            self.skipTest("Paramiko is not available")
        from plumbum.machines.paramiko_machine import ParamikoMachine
        
        rem = ParamikoMachine("localhost", missing_host_policy = paramiko.AutoAddPolicy())
        with DeployedServer(rem) as dep:
            conn = dep.classic_connect()
            print (conn.modules.sys)
            func = conn.modules.os.getcwd
            print (func())

        try:
            func()
        except EOFError:
            pass
        else:
            self.fail("expected an EOFError")


if __name__ == "__main__":
    unittest.main()



from __future__ import with_statement
import subprocess
import sys
import os
import rpyc
import unittest
import tempfile
import shutil
import traceback
from rpyc.experimental.splitbrain import splitbrain, localbrain, enable_splitbrain, disable_splitbrain


if not hasattr(unittest.TestCase, "assertIn"):
    unittest.TestCase.assertIn = lambda self, member, container, msg = None: self.assertTrue(member in container, msg)
if not hasattr(unittest.TestCase, "assertNotIn"):
    unittest.TestCase.assertNotIn = lambda self, member, container, msg = None: self.assertFalse(member in container, msg)

def b(st):
    if sys.version_info[0] >= 3:
        return bytes(st, "latin-1")
    else:
        return st

class SplitbrainTest(unittest.TestCase):
    def setUp(self):
        enable_splitbrain()
        server_file = os.path.join(os.path.dirname(os.path.abspath(__file__)), "..", "bin", "rpyc_classic.py")
        self.proc = subprocess.Popen([sys.executable, server_file, "--mode=oneshot", "--host=localhost", "-p0"],
            stdin = subprocess.PIPE, stdout = subprocess.PIPE, stderr = subprocess.PIPE)
        line = self.proc.stdout.readline().strip()
        if not line:
            print (self.proc.stderr.read())
            self.fail("server failed to start")
        self.assertEqual(line, b("rpyc-oneshot"), "server failed to start")
        host, port = self.proc.stdout.readline().strip().split(b("\t"))
        self.conn = rpyc.classic.connect(host, int(port))

    def tearDown(self):
        self.conn.close()
        disable_splitbrain()

    def test(self):
        here = os.getcwd()
        mypid = os.getpid()
        with open("split-test.txt", "w") as f:
            f.write("foobar")
        with splitbrain(self.conn):
            try:
                path = tempfile.mkdtemp()

                import email

                self.assertNotIn("stale", repr(email))

                os.chdir(path)
                hispid = os.getpid()
                self.assertNotEqual(mypid, hispid)
                here2 = os.getcwd()
                self.assertNotEqual(here, here2)
                self.assertFalse(os.path.exists("split-test.txt"))
                with open("split-test.txt", "w") as f:
                    f.write("spam")

                with localbrain():
                    self.assertEqual(os.getpid(), mypid)
                    with open("split-test.txt", "r") as f:
                        self.assertEqual(f.read(), "foobar")

                try:
                    def f():
                        g()
                    def g():
                        h()
                    def h():
                        open("crap.txt", "r")
                    f()
                except IOError:
                    with localbrain():
                        tbtext = "".join(traceback.format_exception(*sys.exc_info()))
                    # pdb.post_mortem(sys.exc_info()[2])
                    self.assertIn("f()", tbtext)
                    self.assertIn("g()", tbtext)
                    self.assertIn("h()", tbtext)
                else:
                    self.fail("This should have raised a IOError")

            finally:
                # we must move away from the tempdir to delete it (at least on windows)
                os.chdir("/")
                shutil.rmtree(path)

        self.assertIn("stale", repr(email))

        self.assertEqual(os.getpid(), mypid)
        self.assertEqual(os.getcwd(), here)

        os.remove("split-test.txt")



if __name__ == "__main__":
    unittest.main()




#server: please, pay attention at line 'print type(product)' inside 
#Order in method add_product 
#we will see Order and not Product class 
#------------------------------------- 
from itertools import count 
from datetime import datetime 
from notify.all import * 
import rpyc 

class Supplier(object): 
    def __init__(self, name): 
        self.name = name 

    def __repr__(self): 
        return 'Supplier: %s' % self.name 

class Product(object): 
    reserveds = Signal() #when a product is set reserved, we throw this signal 
    exposed_reserveds = reserveds 

    def __init__(self, name, supplier, id): 
        self.name = name 
        self.supplier = supplier 
        self.id = id 
        self.reserved = 'N' 

    def set_reserved (self): 
        self.reserved = 'Y' 
        Product.reserveds(self)  #notify the observers 

    def __repr__(self): 
        return '<Product: %s, %s, Id: %s>' % (self.name, self.supplier, self.id) 

class Store(object): 
    def __init__(self, name): 
        self.name = name 
        self._products = [] 

    def input (self, product): 
        self._products.append(product) 

    def output (self, product_name, supplier): 
        for i, p in enumerate(self._products): 
            if p.name == product_name and p.supplier == supplier: 
                ret = self._products.pop(i) 
                ret.set_reserved() 
                return ret 
        raise Exception('There is no: %s, %s' % (product_name, supplier)) 

    exposed_output = output 

    def __repr__(self): 
        ret = 'Store: %s\n' % self.name 
        for p in self._products: 
            ret += '%s\n' % p 
        return ret 

class Client(object): 
    def __init__(self, name): 
        self.name = name 

    def __repr__(self): 
        return 'Client: %s' % self.name 

class Order(object): 
    new_id = count() 

    def __init__(self, client): 
        self.id = Order.new_id.next() 
        self.client = client 
        self.products = [] 
        self.datetime = datetime.now() 

    def add_product(self, product): 
        print type(product)        #  <--------- look at the output: it is not Product but Order class
        print "!!", product 
        self.products.append(product) 

    exposed_add_product = add_product 

    def __repr__(self): 
        ret = ['<Order: %d, %s, Date: %s>' % (self.id, self.client, 
                         self.datetime.strftime('%d-%m-%Y %H:%M'))] 
        for p in self.products: 
            ret.append('\t%s' % p) 
        return '\n'.join(ret) 

#creating some objects 

estrella_levante = Supplier('Estrella Levante') 
store = Store('Trastienda') 
client = Client('Miguel Angel') 

for i in range(5): 
    store.input(Product('quinto', estrella_levante, '#%d' % i)) 
    store.input(Product('tercio', estrella_levante, '#%d' % i)) 

class MyService(rpyc.Service): 
    exposed_store = store 
    exposed_Order = Order 
    exposed_estrella_levante = estrella_levante 
    exposed_client = client 
    exposed_Product = Product 

    def __init__(self, *args): 
        rpyc.Service.__init__(self, *args) 
        self.signal_handler = []        #when registering signal of py-notify with the handler 

    def on_disconnect(self): 
        for s, h in self.signal_handler: 
            s.disconnect(h) 

    # we register py-notify signal with the handler this way, so when 
    # closing we disconnect every connection 
    def exposed_connect(self, signal, handler): 
        handler =  rpyc.async(handler) 
        self.signal_handler.append((signal, handler)) 
        signal.connect(handler) 

    def exposed_disconnect(self, signal, handler): 
        handler =  rpyc.async(handler) # py-notify disconnects passing 
                                       # as argument the handler object, as we connected 
        try: 
            self.signal_handler.remove((signal, handler)) 
            signal.disconnect(handler) 
        except: 
            pass 

if __name__ == "__main__": 
    from rpyc.utils.server import ThreadedServer 
    t = ThreadedServer(MyService, port = 18861) 
    t.start()

